import prisma from '../utilities/Prisma' // Import the global Prisma instance
import { Character } from '@prisma/client'

class CharacterRepository {
  async getByUserId(userId: number): Promise<Character[] | null> {
    try {
      return await prisma.character.findMany({
        where: {
          userId
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to get character by user ID: ${error.message}`)
    }
  }

  async getByUserAndId(userId: number, characterId: number): Promise<Character | null> {
    try {
      return await prisma.character.findFirst({
        where: {
          userId,
          id: characterId
        },
        include: {
          zone: true
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to get character by user ID and character ID: ${error.message}`)
    }
  }

  async getById(id: number): Promise<Character | null> {
    try {
      return await prisma.character.findUnique({
        where: {
          id
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to get character by ID: ${error.message}`)
    }
  }

  async create(userId: number, name: string, role: 'player'): Promise<Character | null> {
    try {
      return await prisma.character.create({
        data: {
          userId,
          name,
          role,
          position_x: 0, // @TODO Set default registration values in the database
          position_y: 0, // @TODO Set default registration values in the database
          rotation: 0, // @TODO Set default registration values in the database
          zoneId: 1 // @TODO Set default registration values in the database
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to create character: ${error.message}`)
    }
  }

  async updatePosition(id: number, position_x: number, position_y: number): Promise<Character | null> {
    try {
      return await prisma.character.update({
        where: {
          id: id
        },
        data: {
          position_x,
          position_y
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to update character: ${error.message}`)
    }
  }

  async delete(id: number): Promise<Character | null> {
    try {
      return await prisma.character.delete({
        where: {
          id
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to delete character: ${error.message}`)
    }
  }

  async deleteByUserIdAndId(userId: number, characterId: number): Promise<Character | null> {
    try {
      return await prisma.character.delete({
        where: {
          userId,
          id: characterId
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to delete character by user ID and character ID: ${error.message}`)
    }
  }

  async getByName(name: string): Promise<Character | null> {
    try {
      return await prisma.character.findFirst({
        where: {
          name
        }
      })
    } catch (error: any) {
      // Handle error
      throw new Error(`Failed to get character by name: ${error.message}`)
    }
  }
}

export default new CharacterRepository()
